/**

  This is a simple User Interface for creating SPKI certificates
  that delegate Java permissions.

  */

import fi.hut.tcm.spki.*;
import fi.hut.tcm.spki.impl.*;
import fi.hut.tcm.repository.*;
import java.security.*;
import java.security.cert.*;
import java.io.*;
import java.util.jar.Manifest;
import java.util.Iterator;
import sun.security.util.PropertyExpander;

public class UI {

  static KeyStore ks;
  static boolean print;
  static String keystorefile;

  public static void main(String[] args)
       throws Exception 
  {
    keystorefile = new String(PropertyExpander.expand("${user.home}/.keystore"));
    SPKICertificateFactory factory = (SPKICertificateFactory) new SpkiCertFactory();
    SPKICertificate certificate;
    String input;
    String input2;
    Subject subjectobj = null;
    PublicKey subject = null;
    byte[] subjectbytes = new byte[0];
    String hashUri = null;
    boolean delegation = false;
    fi.hut.tcm.spki.Tag tag = null;
    BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
    boolean manifest = false;
    String outputFile = null;
    String passwd = null;
    String keypasswd = null;
    String deleg = null;
    String iss = null;
    String subj = null;
    String type = null;
    String target = new String("");
    String action = new String("");
    boolean hastag = false;
    boolean cont;
    boolean debug = false;
    print = false;

    if ((args.length == 1) && (args[0].equals("-help"))) {
      System.out.println("Usage: java UI [options]");
      System.out.println("\nOptions:\n[-storepass <passwd>]");
      System.out.println("[-keypass <keypasswd>]");
      System.out.println("[-manifest <outputfilename>]");
      System.out.println("[-issuer <keyalias>]");
      System.out.println("[-subject <keyalias>]");
      System.out.println("[-nodeleg] | [-deleg]");
      System.out.println("[-type <full permission class name>]");
      System.out.println("[-target <targetstring>]");
      System.out.println("[-action <actionstring>]");
      System.out.println("[-verbose]");
      return;
    }

    int index = 0;
    while (index < args.length) {
      if (args[index].equals("-manifest")) {
	if (index != args.length - 1) {
	  manifest = true;
	  index++;
	  outputFile = args[index];
	}
      }
      else if (args[index].equals("-storepass")) {
	if (index != args.length - 1) {
	  index++;
	  passwd = args[index];
	}
      }
      else if (args[index].equals("-keypass")) {
	if (index != args.length - 1) {
	  index++;
	  keypasswd = args[index];
	}
      }
      else if (args[index].equals("-nodeleg")) {
	deleg = "f";
      }
      else if (args[index].equals("-deleg")) {
	deleg = "t";
      }
      else if (args[index].equals("-issuer")) {
	if (index != args.length - 1) {
	  index++;
	  iss = args[index];
	}
      }
      else if (args[index].equals("-subject")) {
	if (index != args.length - 1) {
	  index++;
	  subj = args[index];
	}
      }
      else if (args[index].equals("-type")) {
	if (index != args.length - 1) {
	  index++;
	  type = args[index];
	  hastag = true;
	}
      }
      else if (args[index].equals("-target")) {
	if (index != args.length - 1) {
	  index++;
	  target = args[index];
	}
      }
      else if (args[index].equals("-action")) {
	if (index != args.length - 1) {
	  index++;
	  action = args[index];
	}
      }
      else if (args[index].equals("-verbose")) {
	print = true;
      }
      index++;
    }

    ks = KeyStore.getInstance("JKS");
    if (passwd == null) {
      System.out.print("\nGive the keystore password: ");
      passwd = reader.readLine();
    }
    ks.load(new FileInputStream(keystorefile), passwd.toCharArray());
    if (debug) {
      System.out.println("Keystore loaded");
    }

    if (iss == null) {
      System.out.print("\nGive the alias of the issuer's key: ");
      iss = reader.readLine();
    }
    if (keypasswd == null) {
      System.out.print("\nGive the password for the issuer's private key: (return if the password is the same) ");
      keypasswd = reader.readLine();
      if (keypasswd.length() == 0)
	keypasswd = passwd;
    }
    PublicKey issuer = fetchKey(iss); 
    PrivateKey ip = fetchPrivateKey(iss, keypasswd);
    KeyPair pair = new KeyPair(issuer, ip);
    if (debug) {
      System.out.println(issuer.toString());
    }
    
    if (!manifest) {
      if (subj == null) {
	System.out.print("\nGive the alias of the subject's key: ");
	input = reader.readLine();
	subject = fetchKey(input); 
      } else {
	subject = fetchKey(subj); 
      }
    } else {
      // read the manifest file and make an obj-hash of it
      FileInputStream streamIn = new FileInputStream("META-INF/MANIFEST.MF");
      subjectbytes = new byte[streamIn.available()];
      streamIn.read(subjectbytes);
      hashUri = "file://jar.manifest/";
      // FIX: add the class file names to the uri string
      Manifest man = new Manifest(new ByteArrayInputStream(subjectbytes));
      Iterator i = man.getEntries().keySet().iterator();
      while (i.hasNext()) {
	hashUri += i.next().toString();
	if (i.hasNext())
	  hashUri += "#";  // separate the names from each other with a #
      }
      subjectobj = new Subject(subjectbytes, hashUri);
    }
    if (debug) {
      //      System.out.println(subject.toString());
    }
    
    if (deleg == null) {
      System.out.print("\nDo you want to allow further delegation of this right: (y/n) ");
      cont = true;
      while (cont) {
	input = reader.readLine();
	if ((input.equals("y")) || (input.equals("Y"))) {
	  delegation = true;
	  cont = false;
	} else if ((input.equals("n")) || (input.equals("N"))) {
	  delegation = false;
	  cont = false;
	} else {
	  System.out.println("Unknown answer: " + input + " Answer y or n");
	}
      }
    } else if (deleg == "t") {
      delegation = true;
    } else if (deleg == "f") {
      delegation = false;
    } else {
      throw new Exception();
    }
    if (debug) {
      System.out.println(delegation);
    }

    if (!hastag) {
      System.out.println("\nGive * or the full class name of the Java permission: ");
      input = reader.readLine();
    
      if (input.equals("*")) {
	tag = makeTagStar();
      } else {
	System.out.println("\nGive the target of the Java permission, if any: ");
	target = reader.readLine();
	if (target.length() > 0) {
	  System.out.println("\nGive the action of the Java permission, if any: ");
	  action = reader.readLine();
	}
	tag = makeJavaTag(input, target, action);
      }
    } else {
      if (type.equals("*")) {
	tag = makeTagStar();
      } else {
	tag = makeJavaTag(type, target, action);
      }
    }
    if (debug) {
      System.out.println(tag.toString());
    }

    // Make the certificate
    if (manifest)
      certificate = factory.makeCert(pair, subjectobj, delegation, tag);
    else
      certificate = factory.makeCert(pair, subject, delegation, tag);

    if (print) {
      System.out.println("The created certificate has the following information: ");
      System.out.println(certificate.toString());
      System.out.print("Is the information correct? (y/n) ");
      cont = true;
      while (cont) {
	input = reader.readLine();
	if ((input.equals("y")) || (input.equals("Y"))) {
	  if (!manifest) 
	    store(certificate);
	  else
	    output(certificate, outputFile);
	  return;
	} else if ((input.equals("n")) || (input.equals("N"))) {
	  return;
	} else {
	  System.out.println("Answer y or n");
	}
      }
    } else {
      if (!manifest) 
	store(certificate);
      else
	output(certificate, outputFile);
      return;
    }
  }

  private static PublicKey fetchKey(String alias) throws Exception {
   
    if (ks.containsAlias(alias)) {
      java.security.cert.Certificate certif = ks.getCertificate(alias);
      return certif.getPublicKey();
    } else {
      throw new Exception("No such alias: " + alias);
    }
  }

  private static PrivateKey fetchPrivateKey(String alias, String password) throws Exception {
   
    if (ks.containsAlias(alias)) {
      return (PrivateKey)ks.getKey(alias, password.toCharArray());
    } else {
      throw new Exception("No such alias: " + alias);
    }
  }

  private static fi.hut.tcm.spki.Tag makeTagStar() {
      fi.hut.tcm.spki.Tag tag = new fi.hut.tcm.spki.impl.Tag();
      TagStar star = new TagStar();
      tag.add(star);
      return tag;
  }

  private static fi.hut.tcm.spki.Tag makeJavaTag(String type, String target, String action) {
    fi.hut.tcm.spki.Tag t = new fi.hut.tcm.spki.impl.Tag();
    TagExpr tag = new TagExpr();
    TagSimple javaTag = new TagSimple();
    TagExpr typeExpr = new TagExpr();
    TagExpr targetExpr = new TagExpr();
    TagExpr actionExpr = new TagExpr();

    TagSimple typeTag = new TagSimple();
    TagSimple targetTag = new TagSimple();
    TagSimple actionTag = new TagSimple();

    TagExpr typeE = new TagExpr();
    TagString typeS = new TagString();
    TagExpr targetE = new TagExpr();
    TagString targetS = new TagString();
    TagExpr actionE = new TagExpr();
    TagString actionS = new TagString();

    // making tag
    javaTag.add(new ByteString("java-permission"));

    // making type
    typeS.add(new ByteString(type));
    typeE.add(typeS);
    typeTag.add(new ByteString("type"));
    typeTag.add(typeE);
    typeExpr.add(typeTag);
    javaTag.add(typeExpr);

    if (target.length() > 0) {
      // making target
      targetS.add(new ByteString(target));
      targetE.add(targetS);
      targetTag.add(new ByteString("target"));
      targetTag.add(targetE);
      targetExpr.add(targetTag);
      javaTag.add(targetExpr);
    }

    if ((action != null) && (action.length() > 0)) {
      // making action
      actionS.add(new ByteString(action));
      actionE.add(actionS);
      actionTag.add(new ByteString("action"));
      actionTag.add(actionE);
      actionExpr.add(actionTag);
      javaTag.add(actionExpr);
    }

    tag.add(javaTag);
    t.add(tag);

    return t;
  }

  private static void store(SPKICertificate certificate) throws Exception {

    SPKICertificateRepository cr = SPKICertificateRepository.getInstance();

    cr.load();
    cr.add(certificate);
    cr.save();

    if (print) {
      System.out.println("Wrote: \n" + certificate.toString());
    }
  }

  private static void output(SPKICertificate certificate, String file) throws Exception {

    FileOutputStream stream = new FileOutputStream(file);
    stream.write(certificate.getEncoded());
    stream.close();
    
    if (print) {
      System.out.println("Wrote: \n" + certificate.toString());
    }
  }

}
