/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  Composite
  Package: spki
  File:    Composite.java
*/

package fi.hut.tcm.spki;
import fi.hut.tcm.spki.*;
import java.util.Iterator;

/**
  Composite
  
  An interface for SPKI components that have an internal structure.
  
  Used for traversing SPKI data structures.
*/
public interface Composite
  extends Component {

  /**
    Add a new component to a composite. Returns the index of the added 
    component. This index number may later change if earlier components are
     removed.
  */
  public abstract int add(Component addThis)
    throws IllegalArgumentException,IllegalStateException ;
  
  
  /**
    Checks if the component to be added is of correct type. Returns true if
     it is, false if it is not. Throws IllegalStateException if this 
    component is of right type, but cannot be added now (there is already a
     max number of such components, for example).
    
    All composites override this to restrict what classes can be added to 
    them.
  */
  public abstract boolean check(Component addition)
    throws IllegalStateException ;
  
  
  /**
    Get returns the wanted child by it's number. If not found, throws 
    ArrayIndexOutOfBoundsException.
  */
  public abstract Component get(int number)
    throws ArrayIndexOutOfBoundsException ;
  
  
  /**
    Returns the first element x that fullfills (x instanceof childClass). 
    This is handy for searching Composites and subclasses of ByteString, 
    but not ByteStrings.
  */
  public abstract Component get(Component childClass);
  
  
  /**
    Removes the first occurrence of this object. Returns true if the 
    argument was a component of this composite; false otherwise. Elements 
    of a higher index are moved one step downwards.
  */
  public abstract boolean remove(Component child);
  
  
  /**
    Returns an iterator over all the children.
  */
  public abstract Iterator iterator();
  
  
  /**
  */
  public abstract int size();
  
  
} // Composite

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
