/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  SPKICertificate
  Package: spki
  File:    SPKICertificate.java
*/

package fi.hut.tcm.spki;
import java.security.cert.Certificate;
import java.security.PublicKey;
import java.net.URL;
import fi.hut.tcm.spki.*;
import java.io.*;

/**
  SPKICertificate
  
  The abstract superclass for all SPKI certificates.
*/
public abstract class SPKICertificate
  extends Certificate implements Serializable {

  /**
  */
  protected SPKICertificate() {
  
  //+ SPKICertificate::SPKICertificate( )
    super("SPKI");
  //- SPKICertificate::SPKICertificate( )
  }
  
  /**
    Returns the certificate version number. 
    
  */
  public abstract int getVersion();
  
  /**
    Sets a display hint for how the SPKI certificate  
    is best displayed. The hint is optional and may be ignored.
    
    The verifier of the certificate should treat the CertDisplay field as a
     comment, i.e. ignore it.
  */
  protected abstract void putCertDisplay(ByteString certDisplay)
    throws NoSuchFieldException,NoSuchMethodException;
  
  /**
    Gets the display hint for how the SPKI certificate  
    is best displayed. The hint is optional and may be ignored.
    
    The verifier of the certificate should treat the CertDisplay field as a
     comment, i.e. ignore it.
  */
  public abstract ByteString getCertDisplay()
    throws NoSuchFieldException,NoSuchMethodException;
  
  /**
  */
  protected abstract void putIssuer(PublicKey issuer)
    throws IllegalStateException,NoSuchMethodException;
  
  /**
  */
  public abstract PublicKey getIssuer()
    throws NoSuchFieldException,NoSuchMethodException;
  
  /**
  */
  protected abstract void putIssuerInfo(URL[] info)
    throws NoSuchFieldException,NoSuchMethodException;
  
  /**
  */
  public abstract URL[] getIssuerInfo()
    throws NoSuchFieldException,NoSuchMethodException;
  
  /**
  */
  protected abstract void putSubject(Object subject)
    throws NoSuchFieldException,NoSuchMethodException;
  
  /**
  */
  public abstract Object getSubject()
    throws NoSuchFieldException,NoSuchMethodException;
  
  /**
  */
  protected abstract void putSubjectInfo(URL[] info)
    throws NoSuchFieldException,NoSuchMethodException;
  
  /**
  */
  public abstract URL[] getSubjectInfo()
    throws NoSuchFieldException,NoSuchMethodException;
  
  /**
  */
  protected abstract void putDeleg(boolean deleg)
    throws IllegalStateException,NoSuchMethodException;
  
  /**
  */
  public abstract boolean getDeleg()
    throws NoSuchFieldException,NoSuchMethodException;
  
  /**
  */
  protected abstract void putValidity(Validity valid)
    throws IllegalStateException,NoSuchMethodException;
  
  /**
  */
  public abstract Validity getValidity()
    throws NoSuchFieldException,NoSuchMethodException;
  
  /**
  */
  protected abstract void putTag(Tag tag)
    throws IllegalStateException,NoSuchMethodException;
  
  /**
  */
  public abstract Tag getTag()
    throws NoSuchFieldException,NoSuchMethodException;
  
  /**
  */
  protected abstract void putComment(ByteString comment)
    throws IllegalStateException,NoSuchMethodException;
  
  /**
  */
  public abstract ByteString getComment()
    throws NoSuchFieldException,NoSuchMethodException;
  
  /**
  */
  public abstract boolean isValid();

  /**
  */
  private void writeObject(java.io.ObjectOutputStream out)
    throws IOException {
      out.defaultWriteObject();
  }

  /**
  */
  private void readObject(java.io.ObjectInputStream in)
    throws IOException, ClassNotFoundException {
      in.defaultReadObject();
  }

} // SPKICertificate

// +*************** UNUSED CODE ***************
//+ SPKICertificate::putTag(Tag)
//- SPKICertificate::putTag(Tag)
//+ SPKICertificate::putDeleg(Boolean)
//- SPKICertificate::putDeleg(Boolean)
//+ SPKICertificate::getTag( )
//- SPKICertificate::getTag( )
//+ SPKICertificate::putValidity(Validity)
//- SPKICertificate::putValidity(Validity)
//+ SPKICertificate::getValidity( )
//- SPKICertificate::getValidity( )
//+ SPKICertificate::getDeleg( )
//- SPKICertificate::getDeleg( )
//+ SPKICertificate::getComment( )
//- SPKICertificate::getComment( )
//+ SPKICertificate::getIssuer( )
//- SPKICertificate::getIssuer( )
//+ SPKICertificate::isValid( )
//- SPKICertificate::isValid( )
//+ SPKICertificate::putIssuer(PublicKey)
//- SPKICertificate::putIssuer(PublicKey)
//+ SPKICertificate::putComment(ByteString)
//- SPKICertificate::putComment(ByteString)
//+ SPKICertificate::putSubject(Object)
//- SPKICertificate::putSubject(Object)
//+ SPKICertificate::getSubject( )
//- SPKICertificate::getSubject( )
//+ SPKICertificate::putTag(Tag)
//- SPKICertificate::putTag(Tag)
//+ SPKICertificate::putDeleg(Boolean)
//- SPKICertificate::putDeleg(Boolean)
//+ SPKICertificate::getTag( )
//- SPKICertificate::getTag( )
//+ SPKICertificate::putValidity(Validity)
//- SPKICertificate::putValidity(Validity)
//+ SPKICertificate::getValidity( )
//- SPKICertificate::getValidity( )
//+ SPKICertificate::getDeleg( )
//- SPKICertificate::getDeleg( )
//+ SPKICertificate::getComment( )
//- SPKICertificate::getComment( )
//+ SPKICertificate::getIssuer( )
//- SPKICertificate::getIssuer( )
//+ SPKICertificate::isValid( )
//- SPKICertificate::isValid( )
//+ SPKICertificate::putIssuer(PublicKey)
//- SPKICertificate::putIssuer(PublicKey)
//+ SPKICertificate::putComment(ByteString)
//- SPKICertificate::putComment(ByteString)
//+ SPKICertificate::putSubject(Object)
//- SPKICertificate::putSubject(Object)
//+ SPKICertificate::getSubject( )
//- SPKICertificate::getSubject( )
// -*************** UNUSED CODE ***************
