package fi.hut.tcm.spki;
import fi.hut.tcm.spki.*;
import fi.hut.tcm.spki.impl.*;
import java.util.ArrayList;
import java.util.Hashtable;
import java.security.Security;
import java.security.PublicKey;
import java.security.MessageDigest;
import java.security.cert.Certificate;
import java.security.cert.CertificateFactory;
import java.security.cert.CertificateEncodingException;
import sun.security.util.ManifestDigester;
import java.io.ObjectInputStream;
import java.io.ByteArrayInputStream;

public class SignatureVerifier {

  private ArrayList certCache;
  private ManifestDigester manDig;
  private String name;
  private byte[] bytes;

  /**
   * Creates a new SignatureVerifier.
   * @param certCache currently ignored
   * @param manDig digest of the manifest file
   * @param name hash table key
   * @param bytes the SPKI certificate(s) to check
   */
  public SignatureVerifier(ArrayList certCache,
                           ManifestDigester manDig,
                           String name,
                           byte[] bytes) {
    this.certCache = certCache;
    this.manDig = manDig;
    this.name = name;
    this.bytes = bytes;
  }

  /**
   * Processes the certificates and adds good ones to a hashtable
   * @param certs the hashtable to add certs to
   */
  public void process(Hashtable hash) 
  throws /*XXX*/Exception {
    boolean debug = false;

    MessageDigest sha1 = MessageDigest.getInstance("SHA");

    sha1.reset();
    byte[] manifestDigest = manDig.manifestDigest(sha1);
    
    /* Parse the certificate and verify signature */
    //    Class xxx = Class.forName("fi.hut.tcm.spki.SPKICertificate");
    ByteArrayInputStream streamIn = new ByteArrayInputStream(bytes); 
    Security.addProvider(new fi.hut.tcm.spki.impl.SPKIProvider());
    CertificateFactory factory = CertificateFactory.getInstance("SPKI");
    Certificate cert = factory.generateCertificate(streamIn);

    cert.verify(cert.getPublicKey());
    if (debug) {
      System.out.println("Signature OK");
    }

    /****
     * Check the manifest digest
     ****/
    // Compare to the certificate subject
    Subject certSubj = (Subject) ((SPKICertificate) cert).getSubject();
    Hash subjhash;
    HashAlgName halgname;
    HashValue hval;
    Uri hname;
    if (!(certSubj.get(0) instanceof ObjHash)) {
      // Subject type inappropriate: certificate ignored
      if (debug) {
	System.out.println("Wrong subject type: " + certSubj.toString());
      }
      return;
    } else {
      try {
	subjhash = (Hash) ((Composite) certSubj.get(0)).get(0);
	halgname = (HashAlgName) subjhash.get(0);
	hval = (HashValue) subjhash.get(1);
	hname = (Uri) subjhash.get(2);
      } catch (IndexOutOfBoundsException e) {
	// Subject type inappropriate: certificate ignored
	if (debug) {
	  System.out.println("Too few fields in the subject: " + certSubj.toString());
	}
	return;
      }
      // get the digest bytes from the certificate
      String valstring = hval.getValueString();
      byte[] digestFromCert = null;
      if ((valstring.startsWith("|")) && (valstring.endsWith("|"))) {
	// BASE64 encoded
	sun.misc.CharacterDecoder decod64 = new sun.misc.BASE64Decoder();
	digestFromCert = decod64.decodeBuffer(valstring.substring(1, valstring.length()));
      } else {
	// value is not BASE64 encoded
	digestFromCert = hval.getValueBytes();
      }

      // FIX: there has to be a better way to transfer the class names...
      // halgname has to be "sha1", hval has to be equal 
      // to the manifest digest, and the hname has to start with "file://manifest/"
      /* XXX doesn't work
	 if (((ByteString)halgname.get(0)).getValueString().equals(new String("sha1")) &&
	 digestFromCert.equals(manifestDigest) &&
	 hname.getValueString().startsWith(new String("file://jar.manifest/"))) {
	 */
      if (((ByteString)halgname.get(0)).getValueString().equals(new String("sha1")) &&
	  hname.getValueString().startsWith(new String("file://jar.manifest/"))) {
	// cert matches the manifest: OK
	if (debug)
	  System.out.println("Hash checked OK");
      } else {
	// cert not issued to this manifest: ignore it
	if (debug)
	  System.out.println("Certificate not issued to a manifest");
	return;
      }
    }

    /***
     * Get the names of classes from the certificate
     */
    String tmpstring = hname.getValueString().substring(20);
    // XXX
    System.out.println(tmpstring);
    String namestring = null;
    String names[] = null;
    while (tmpstring.length() > 0) {
      int index = tmpstring.lastIndexOf("#");
      if (index == -1) {
	// only one class name left
	namestring = tmpstring;
	tmpstring = new String();
      } else {
	// The name of the class is to the right of the slash, 
	// the rest of the string to check is to the left
	namestring = tmpstring.substring(index + 1, tmpstring.length());
	tmpstring = tmpstring.substring(0, index);
      }
      // XXX Shouldn't we check the name in any way?

      if (names == null) {
	names = new String[1];
      } else {
	String newnames[] = new String[names.length + 1];
	System.arraycopy(names, 0, newnames, 0, names.length);
	names = newnames;
      }
      names[names.length - 1] = namestring;
    }

    /****
     * Check the digests of the classes
     ****/
    for (int i = 0; i < names.length; i++) {
      sha1.reset();
      byte[] digest = manDig.get(names[i], false).digest(sha1);
      // Check per class digest
      // throw XXXException if fails

      // Not done at the moment!!!
    }

    /***
     * Add the relevant certificate(s) to the classes
     **/
    for (int i = 0; i < names.length; i++) {
      String name = names[i];
      Certificate[] certs = (Certificate[])hash.get(name);
      if (certs == null) {
        certs = new Certificate[1];
      } else {
        Certificate[] newcerts = new Certificate[certs.length + 1];
        System.arraycopy(certs, 0, newcerts, 0, certs.length);
        certs = newcerts;
      }
      certs[certs.length - 1] = cert;
      hash.put(name, certs);
    }
  }
  
}
