/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  Cert
  Package: impl
  File:    Cert.java
*/

package fi.hut.tcm.spki.impl;
import fi.hut.tcm.spki.*;
import fi.hut.tcm.spki.format.*;

/**
  Cert
  
  Cert-class contains the data of a single certificate. Consists of Issuer,
   Subject, Deleg and a Tag in that order, followed by the optional fields 
  Valid and Comment in that order.
*/
public class Cert
  extends Composite
  implements GenericFormat, Format, fi.hut.tcm.spki.Composite, Component, SeqEnt {

  /**
  */
  public Cert() {
  
  //+ Cert::Cert( )
    super();
  //- Cert::Cert( )
  }
  
  /**
  */
  public boolean check(Component addition)
    throws IllegalStateException  {
  
  //+ Cert::check(Component)
      if (!(addition instanceof CertDisplay) &&
	  !(addition instanceof Issuer) &&
	  !(addition instanceof IssuerInfo) &&
	  !(addition instanceof Subject) &&
	  !(addition instanceof SubjectInfo) &&
	  !(addition instanceof Deleg) &&
	  !(addition instanceof Tag) &&
	  !(addition instanceof Valid) &&
	  !(addition instanceof Comment))
	return false;
      
      if ((addition instanceof CertDisplay) && (children.size() == 0))
	return true;
      else if ((addition instanceof Issuer) && 
	       ((children.size() == 0) || (children.size() == 1)))
	return true;
      else if ((addition instanceof IssuerInfo) && 
	       (get(children.size() - 1) instanceof Issuer))
	return true;
      else if (((addition instanceof Subject) && 
		((get(children.size() - 1) instanceof Issuer) || 
		 (get(children.size() - 1) instanceof IssuerInfo))))
	return true;
      else if ((addition instanceof SubjectInfo) && 
	       (get(children.size() - 1) instanceof Subject))
	return true;
      else if ((addition instanceof Deleg) && 
	       ((get(children.size() - 1) instanceof Subject) || 
		(get(children.size() - 1) instanceof SubjectInfo)))
	return true;
      else if ((addition instanceof Tag) && 
	       ((get(children.size() - 1) instanceof Subject) || 
		(get(children.size() - 1) instanceof SubjectInfo) || 
		(get(children.size() - 1) instanceof Deleg)))
	return true;
      else if ((addition instanceof Valid) && 
	       (get(addition) == null) &&
	       ((get(children.size() - 1) instanceof Tag) ||
		(get(children.size() - 1) instanceof Valid)))
	return true;
      else if ((addition instanceof Comment) && 
	       ((get(children.size() - 1) instanceof Valid) ||
		(get(children.size() - 1) instanceof Tag)))
	return true;
      
      throw new IllegalStateException("Addition: " + addition + 
				      " Children: " + children.size());
  //- Cert::check(Component)
  }
  
  /**
  */
  public boolean isSingleCert() {
  
  //+ Cert::isSingleCert( )
      return true;
  //- Cert::isSingleCert( )
  }
  
  /**
  */
  public void accept(Visitor visitor) {
  
  //+ Cert::accept(Visitor)
      visitor.at(this);
  //- Cert::accept(Visitor)
  }
  
  /**
  */
  public String toString() {
  
  //+ Cert::toString( )
    String result = new String("(cert");
    for (int i = 0; i < size(); i++)
      result = new String(result + " " + get(i).toString());
    return new String(result + ")");
  //- Cert::toString( )
  }
  
} // Cert

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
