/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  Composite
  Package: impl
  File:    Composite.java
*/

package fi.hut.tcm.spki.impl;
import fi.hut.tcm.spki.*;
import java.util.Iterator;
import java.util.Vector;

/**
  Composite
  
  Data structure part, which has child classes.
*/
public abstract class Composite
  extends Object
  implements fi.hut.tcm.spki.Composite, Component {

  /**
  */
  protected Vector children;
  
  /**
  */
  public Composite() {
  
  //+ Composite::Composite( )
    super();
    children = new Vector();
  //- Composite::Composite( )
  }
  
  /**
  */
  public void accept(Visitor visitor) {
  
  //+ Composite::accept(Visitor)
    visitor.at(this);
  //- Composite::accept(Visitor)
  }
  
  /**
    Add a new component to a composite. Returns the index of the added 
    component. This index number may later change if earlier components are
     removed.
  */
  public int add(Component addThis)
    throws IllegalArgumentException,IllegalStateException  {
  
  //+ Composite::add(Component)
      if (check(addThis)) {
	children.addElement(addThis);
	return children.size() - 1;
      }
      else
	throw new IllegalArgumentException("Check failed: " + addThis);
  //- Composite::add(Component)
  }
  
  /**
    Checks if the component to be added is of correct type. Returns true if
     it is, false if it is not. Throws IllegalStateException if this 
    component is of right type, but cannot be added now (there is already a
     max number of such components, for example).
    
    All composites override this to restrict what classes can be added to 
    them.
  */
  public abstract boolean check(Component addition)
    throws IllegalStateException ;
  
  
  /**
    Get returns the wanted child by it's number. If not found, throws 
    ArrayIndexOutOfBoundsException.
  */
  public Component get(int number)
    throws ArrayIndexOutOfBoundsException  {
  
  //+ Composite::get(int)
      return (Component) children.elementAt(number);
  //- Composite::get(int)
  }
  
  /**
    This function returns the first child x that fullfills (x instanceof 
    childClass). This is handy for searching Composites and subclasses of 
    ByteString, but not ByteStrings.
  */
  public Component get(Component childClass) {
  
  //+ Composite::get(Component)
      for (int i = 0; i < children.size(); i++)
	if (childClass.getClass().isInstance(children.elementAt(i)))
	  return (Component) children.elementAt(i);
      return null;
  //- Composite::get(Component)
  }
  
  /**
    Removes the first occurrence of this object. Returns true if the 
    argument was a component of this composite; false otherwise. Elements 
    of a higher index are moved one step downwards.
  */
  public boolean remove(Component child) {
  
  //+ Composite::remove(Component)
       return children.removeElement(child);
  //- Composite::remove(Component)
  }
  
  /**
    GetEnum returns enumeration of all the children.
  */
  public Iterator iterator() {
  
  //+ Composite::iterator( )
    return children.iterator();
  //- Composite::iterator( )
  }
  
  /**
  */
  public int size() {
  
  //+ Composite::size( )
    return children.size();
  //- Composite::size( )
  }
  
  /**
  */
  public String toString() {
  
  //+ Composite::toString( )
      String result;

      result = "(";
      for (int i = 0; i < size(); i++) {
	result += children.elementAt(i).toString();
	if (i < (size() - 1))
	  result += " ";
      }
      result += ")";
      return result;
  //- Composite::toString( )
  }
  
} // Composite

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
