/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  Issuer
  Package: impl
  File:    Issuer.java
*/

package fi.hut.tcm.spki.impl;
import fi.hut.tcm.spki.*;
import java.security.PublicKey;

/**
  Issuer
  
  The information about certificate issuer. This may be either a key or a 
  fully qualified name. No children means that issuer is "self".
*/
public class Issuer
  extends Composite {

  /**
  */
  public Issuer() {
  
  //+ Issuer::Issuer( )
    super();
  //- Issuer::Issuer( )
  }
  
  /**
  */
  public Issuer(PublicKey key) {
  
  //+ Issuer::Issuer(PublicKey)
    add(new PubKey(key));
  //- Issuer::Issuer(PublicKey)
  }
  
  /**
  */
  public boolean check(Component addition)
    throws IllegalStateException  {
  
  //+ Issuer::check(Component)
      if (!(addition instanceof Name) &&
	  !(addition instanceof SpkiPrincipal))
	return false;
      if (children.size() == 0)
	return true;
      throw new IllegalStateException();
  //- Issuer::check(Component)
  }
  
  /**
  */
  public void accept(Visitor visitor) {
  
  //+ Issuer::accept(Visitor)
    visitor.at(this);
  //- Issuer::accept(Visitor)
  }
  
  /**
  */
  public PublicKey getPublicKey()
    throws IllegalStateException  {
  
  //+ Issuer::getPublicKey( )
      PubKey pk = (PubKey) get(new PubKey());
      if (pk == null)
	throw new IllegalStateException("This issuer is not a public key.");
      else
	return pk.getPublicKey();
  //- Issuer::getPublicKey( )
  }
  
  /**
  */
  public String toString() {
  
  //+ Issuer::toString( )
      String result;

      result = "(issuer ";
      for (int i = 0; i < size(); i++) {
	result += children.elementAt(i).toString();
	if (i < (size() - 1))
	  result += " ";
      }
      result += ")";
      return result;
  //- Issuer::toString( )
  }
  
} // Issuer

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
