/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  OnlineType
  Package: impl
  File:    OnlineType.java
*/

package fi.hut.tcm.spki.impl;
import fi.hut.tcm.spki.*;

/**
  OnlineType
  
  Indicates the type of revalidation. The possible values are "crl", "reval
  " and "one-time".
*/
class OnlineType
  extends ByteString {

  /**
    Creates an OnlineType object with the default value
    "crl".
  */
  public OnlineType() {
  
  //+ OnlineType::OnlineType( )
    super();
  //- OnlineType::OnlineType( )
  }
  
  /**
    Creates a new OnlineType object with the value
    given in the value string. If the value is illegal, i.e.,
    not one of "crl", "reval" and "one-time", the default
    of "crl" will be used.
  */
  public OnlineType(String value) {
  
  //+ OnlineType::OnlineType(String)
    super();
    try {
      if (value.equals("crl") || 
	  value.equals("reval") || 
	  value.equals("one-time"))
	super.setValue(value);
      else
	super.setValue("crl");
    } catch (Exception e) {
      System.err.println(e.toString()); // Never reached.
    }
  //- OnlineType::OnlineType(String)
  }
  
  /**
  */
  public String toString() {
  
  //+ OnlineType::toString( )
    return super.toString();
  //- OnlineType::toString( )
  }
  
  /**
  */
  public boolean equals(Object other) {
  
  //+ OnlineType::equals(Object)
    if ((other instanceof OnlineType) && 
	(((OnlineType)other).getValueString().equals(getValueString())) && 
	(((OnlineType)other).getDisplayHintString().equals(getDisplayHintString())))
      return true;
    else 
      return false;
  //- OnlineType::equals(Object)
  }
  
  /**
  */
  public void setValue(byte[] string)
    throws SPKIException  {
  
  //+ OnlineType::setValue(byte[])
    String value = new String(string);
    if (value.equals("crl") || 
	value.equals("reval") || 
	value.equals("one-time"))
      super.setValue(value);
    else
      throw new SPKIException("Illegal value " + string);
  //- OnlineType::setValue(byte[])
  }
  
  /**
  */
  public void setValue(String string)
    throws SPKIException   {
  
  //+ OnlineType::setValue(String)
    if (string.equals("crl") || 
	string.equals("reval") || 
	string.equals("one-time"))
      super.setValue(string);
    else
      throw new SPKIException("Illegal value " + string);
  //- OnlineType::setValue(String)
  }
  
  /**
  */
  public void accept(Visitor visitor) {
  
  //+ OnlineType::accept(Visitor)
    visitor.at(this);
  //- OnlineType::accept(Visitor)
  }
  
} // OnlineType

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
