/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  PubSigAlgId
  Package: impl
  File:    PubSigAlgId.java
*/

package fi.hut.tcm.spki.impl;
import fi.hut.tcm.spki.*;

/**
  PubSigAlgId
  
  The identifier of an Asymmetric Key -based signature algorithm. Contains 
  either a ByteString or an Uri. The possible string values are "rsa-
  pkcs1-md5", "rsa-pkcs1-sha1" and "dsa-sha1".
*/
public class PubSigAlgId
  extends Composite {

  /**
  */
  public PubSigAlgId() {
  
  //+ PubSigAlgId::PubSigAlgId( )
    super();
  //- PubSigAlgId::PubSigAlgId( )
  }
  
  /**
  */
  public boolean check(Component addition)
    throws IllegalStateException  {
  
  //+ PubSigAlgId::check(Component)
      if (!(addition instanceof Uri) &&
	  !(addition instanceof ByteString &&
	    !addition.getClass().getSuperclass().getName().equals
	    ((new ByteString()).getClass().getName())))
	return false;
      if (addition instanceof ByteString) {
	String s = ((ByteString) addition).getValueString();
	if (!s.equals("dsa-sha1") &&
	    !s.equals("rsa-pkcs1-sha1") &&
	    !s.equals("rsa-pkcs1-md5"))
	  return false;
      }
      if (children.size() == 0)
	return true;
      throw new IllegalStateException();
  //- PubSigAlgId::check(Component)
  }
  
  /**
  */
  public void accept(Visitor visitor) {
  
  //+ PubSigAlgId::accept(Visitor)
      visitor.at(this);
  //- PubSigAlgId::accept(Visitor)
  }
  
  /**
  */
  public String toString() {
  
  //+ PubSigAlgId::toString( )
    String result = new String();
    for (int i = 0; i < size(); i++) {
      result += children.elementAt(i).toString();
      if (i < (size() - 1))
	result += " ";
    }
    return result;
  //- PubSigAlgId::toString( )
  }
  
} // PubSigAlgId

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
