import fi.hut.tcm.spki.*;
import fi.hut.tcm.spki.impl.*;
import fi.hut.tcm.policy.*;
import fi.hut.tcm.repository.*;
import java.security.Security;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.KeyStore;
import java.security.PublicKey;
import java.security.PrivateKey;
import java.security.Provider;
import java.security.Policy;
import java.security.CodeSource;
import java.security.PermissionCollection;
import java.security.cert.*;
import java.net.URL;
import java.io.*;
import sun.security.util.PropertyExpander;
import java.util.PropertyPermission;

class testi2 {

  public final static void main(String[] args) throws Exception {

    System.out.println("* Installing provider...");
    Security.addProvider(new fi.hut.tcm.spki.impl.SPKIProvider());
    Provider prov = Security.getProvider("TeSSA.tcm.hut.fi SPKIProvider");
    System.out.println("Provider name: " + prov.getName());
    System.out.println("Provider version: " + prov.getVersion());
    System.out.println("Provider info: " + prov.getInfo());

    System.out.println("* Making CertificateFactory...");
    CertificateFactory f = CertificateFactory.getInstance("SPKI");
    System.out.println("Factory: " + f.toString());
    
    System.out.println("* Making Keys...");
    KeyPairGenerator keygen = KeyPairGenerator.getInstance("DSA");
    keygen.initialize(1024);
    KeyPair kp1 = keygen.genKeyPair();
    KeyPair kp2 = keygen.genKeyPair();
    System.out.println("Public key 1: " + kp1.getPublic().toString());
    System.out.println("Public key 2: " + kp2.getPublic().toString());

    System.out.println("* Making Certificate...");
    fi.hut.tcm.spki.Tag tag = new fi.hut.tcm.spki.impl.Tag();
    TagStar star = new TagStar();
    tag.add(star);
    SpkiCertFactory f2 = new SpkiCertFactory();
    SPKICertificate cert = f2.makeCert(kp1, kp2.getPublic(), false, tag);
    System.out.println("Certificate: " + cert.toString());
    
    System.out.println("* Writing Certificate...");
    FileOutputStream out = new FileOutputStream("testi2.tmp");
    out.write(cert.getEncoded());
    out.close();

    System.out.println("* Reading Certificate...");
    FileInputStream in = new FileInputStream("testi2.tmp");
    Certificate c = f.generateCertificate(in);
    System.out.println("Result: " + c.toString());

    System.out.println("* Comparing keys...");
    if (c.getPublicKey().toString().equals(kp1.getPublic().toString()))
      System.out.println("ok");
    else
      System.out.println("key mismatch");
  }

}
